<?php
/**
 *
 * @package MediaWiki
 * @subpackage SpecialPage
 */

/**
 *
 * @package MediaWiki
 * @subpackage SpecialPage
 */
 

class IntersectPage extends PageQueryPage {
	var $request, $par;

	function IntersectPage( &$request, $cats ) {
		global $wgUser;
		$this->request =& $request;
		$this->skin =& $wgUser->getSkin();
		$this->cats = $cats;
  }

	function getName() {
		return "Intersect";
	}

	function sortDescending() {
		return false;
	}

	function isExpensive() {
		return true;
	}
	
	function isSyndicated() {
	  return false; 
	}

	function getSQL() {
		$dbr =& wfGetDB( DB_SLAVE );
		extract( $dbr->tableNames( 'page', 'categorylinks' ) );
		
		$sql = "SELECT 'Intersect'     AS type,
      		          page_namespace AS namespace,
      		          page_title     AS title,
      		          page_title     AS value
      			FROM $page ";
		
		$i = 0;
		$where = null;
		// TODO: maybe use /=talk/ (etc) to search a namespace
		foreach ($this->cats as $cat) if ($cat != null) {
		  // functionality for 'not this cat', but doesn't fit the GUI well
		  /*if ($cat{0} == '-') {
        $sql .= "LEFT OUTER JOIN $categorylinks AS c".$i;
	      $sql .= " ON page_id=c" .$i. ".cl_from";
	      $sql .= " AND c" .$i. ".cl_to = '" . substr ($cat, 1) . "' ";
	      if ($where != null) $where .= " AND ";
	      $where .= "c" .$i. ".cl_to IS NULL ";
		  } else */ {
        $sql .= "INNER JOIN $categorylinks AS c".$i;
	      $sql .= " ON page_id=c" .$i. ".cl_from";
	      $sql .= " AND c" .$i. ".cl_to = '" .$cat. "' ";
	    }
		  $i ++;
		}
		if ($where != null) {
		  $sql .= " WHERE " .$where;
		}

		return $sql;
	}
}

/**
 * Constructor
 */
function wfSpecialIntersect($par = NULL) {
	list( $limit, $offset ) = wfCheckLimits();

  global $wgRequest;
  global $wgOut;
  global $HTTP_GET_VARS;
  global $wgIntersectVerbose;
  
  //R/ take information from HTTP get, or from parameter
  if ($HTTP_GET_VARS["cats"] != null) {
    $cats = $HTTP_GET_VARS["cats"];
  } else if ($par != null) {
    $cats = explode ("/", rtrim ($par, '/'));
  }

  // display header and list of categories
  $count = 0;
  if ($wgIntersectVerbose != true) {
    
    // terse mode - a short list of categories
    $s = (wfMsg( 'intersectheader' ));
    if ($cats != null) {
      foreach ($cats as $cat) if ($cat != null) {
        $count ++;
        $s .= $i;
        $i  = "; ";
        $s .= '[[:'	. Title::makeTitle (NS_CATEGORY, ltrim ($cat, '-'))->getPrefixedText() . ']]';
      }
    }
    $wgOut->addWikiText ($s . ".");
  } else {
  
    // verbose mode - browser with pulldown menus
    $wgOut->addWikiText (wfMsg( 'intersectheader' ));
    $dbr =& wfGetDB( DB_SLAVE );
    $fname = "Special:Intersect";
		extract( $dbr->tableNames( 'page', 'categorylinks' ) );

    $s = "<form action='" . Title::makeTitle( NS_SPECIAL, 'Intersect' )->getLocalUrl() . "' method='get'>";
    if ($cats != null) {
      foreach ($cats as $cat) if ($cat != null) {
        $count ++;
        $s .= "\n<select name='cats[]'>";
        $t = "";

        $parent = $dbr->select( array ('page', 'categorylinks'), 'cl_to',
          "page_title = '" . $cat . "' AND page_id = cl_from AND page_namespace = " . NS_CATEGORY, $fname );
    		if ( $parent != null ) {
          while ($row = $dbr->fetchRow($parent)) {
            $t .= "\n<option>" . $row[0];
          }
          if ($t != null) {
      		  $s .= "\n<optgroup label='" . wfMsg( 'intersectparent' ) . "'>" . $t . "\n</optgroup>";
      		}
        }
        $s .= "\n<option selected>" . $cat . "\n<option>";
        $t = "";
        
        $child = $dbr->select( array ('page', 'categorylinks'), 'page_title',
          "cl_to = '" . $cat . "' AND page_id = cl_from AND page_namespace = " . NS_CATEGORY, $fname );
    		if ( $child != null ) {
          while ($row = $dbr->fetchRow($child)) {
            $t .= "\n<option>" . $row[0];
          }
          if ($t != null) {
      		  $s .= "\n<optgroup label='" . wfMsg( 'intersectsubcat' ) . "'>" . $t . "\n</optgroup>";
      		}
        }
        
        $s .= "\n</select> ";
      }
    }
    if ($count == 0) {
      $s .= "<input type='text' name='cats[]'> ";
    }
    $s .= "<input type='text' name='cats[]'>";
    $s .= "<input type='submit' value='" . wfMsg( 'intersectsubmit' ) . "'></form>";
    $wgOut->addHtml ($s);
  }

  // intersect query is only meaningful if we have at least two cats
  if ($count < 2) {
		$wgOut->addWikiText (wfMsg ( 'intersecterror' ));
    return null;
  } else {
	  $lpp = new IntersectPage($wgRequest, $cats);
	  return $lpp->doQuery( $offset, $limit );
	}
}

?>
